package com.adaptrex.core.config;

import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.servlet.annotation.WebListener;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adaptrex.core.service.CdiService;
import com.fasterxml.jackson.databind.ObjectMapper;

@ApplicationScoped
public class ConfigListener implements ServletContextListener {
	
	private Logger log = LoggerFactory.getLogger(ConfigListener.class);
	
	@SuppressWarnings("unchecked")
	@Override
	public void contextInitialized(ServletContextEvent event) {
		Map<String,Object> adaptrexConfigMap = null;
		Map<String,Object> contextConfigMap = new HashMap<String,Object>();
		
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		
		/*
		 * Get the application specific config
		 */
		InputStream contextFile = event.getServletContext().getResourceAsStream("/WEB-INF/adaptrex.webapp.config");
		try {
			contextConfigMap.putAll(new ObjectMapper().readValue(contextFile, Map.class));
		} catch (Exception e) {
			log.info("Could not load adaptrex.webapp.config for '" + event.getServletContext().getContextPath() + "'");
		}
		
		
		ClassLoader acl = cl.getParent();
		InputStream adaptrexFile = acl.getResourceAsStream("adaptrex.config");
		try {
			adaptrexConfigMap = new ObjectMapper().readValue(adaptrexFile, Map.class);
			
			/*
			 * Initialize the global adaptrex configuration
			 */
			AdaptrexConfig adaptrexConfig = CdiService.getAdaptrexConfig();
			adaptrexConfig.set(adaptrexConfigMap);
			adaptrexConfig.getMap().remove("webapps");
			
			/*
			 * Initialize the webapp configuration specified in the adaptrex config
			 */
			for (Map<String,Object> contextConfig : (List<Map<String,Object>>) adaptrexConfigMap.get("webapps")) {
				String path = contextConfig.get("contextPath") == null ? "" : (String) contextConfig.get("contextPath");
				if (event.getServletContext().getContextPath().replace("/", "").equals(path)) {
					contextConfigMap.putAll(contextConfig);
					break;
				}
			}
			
		} catch (Exception e) {
			log.warn("Could not load adaptrex.config");
		}
		
		CdiService.getContextConfig().set(contextConfigMap);
	}

	@Override
	public void contextDestroyed(ServletContextEvent event) {
		
	}
}
