package com.adaptrex.core.ext.data;

import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adaptrex.core.persistence.AdaptrexPersistence;
import com.adaptrex.core.service.CdiService;
import com.adaptrex.core.utilities.StringUtilities;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;

/*
 * TODO: Still has JPA specific code!
 */
public class Association {

	private static AdaptrexPersistence persistence = CdiService.getAdaptrexPersistence();
	
	private Logger 	log = LoggerFactory.getLogger(Association.class);
	
	private Config 	config;
	
	private ModelDefinition	ownerModelDefinition;
	private Class<?> 		ownerModelClazz;
	
	private ModelDefinition associatedModelDefinition;
	private Class<?>		associatedModelClazz;
	
	/*
	 * Ext options
	 */
	private String	type;
	private String 	modelName;
	private String	foreignKey;
	private String	name;
	
	/*
	 * From the "rules"
	 * TODO: Reevaluate this
	 */
	private String	associationKey;
	

	/*
	 * Create Return Association
	 */
	public Association(String type, String model, String name) throws ClassNotFoundException {
		this.modelName = model;
		this.type = type;
		this.name = name;
		if (this.type.equals("belongsTo")) {
			this.foreignKey = name + "Id";
		}
	}
	
	/*
	 * Create Association
	 * TODO: Do we need to test the getters and setters, or do the persistence calls
	 * handle it all for us
	 */
	public Association(ModelDefinition ownerModelDefinition, String associatedModelName) throws ClassNotFoundException {
		this.ownerModelDefinition = ownerModelDefinition;
		config = ownerModelDefinition.getConfig();
		this.ownerModelClazz = ownerModelDefinition.getClazz();

		String associatedModelPrefix = "";
		if (associatedModelName.contains(".")) {
			String[] associatedModelParts = associatedModelName.split("\\.");
			associatedModelPrefix = associatedModelParts[0];
			associatedModelName = associatedModelParts[1];
		}
		
		/*
		 * Make sure we have an association between the models
		 */
		Method getter = null;
		try {
			getter = ownerModelClazz.getMethod("get" + associatedModelName);
		} catch (Exception e) {}
		if (getter == null) {
			associatedModelName = StringUtilities.pluralize(associatedModelName);
			try {
				getter = ownerModelClazz.getMethod("get" + associatedModelName);
			} catch (Exception e) {}
		}
		if (getter == null) {
			log.warn("Couldn't find association from " + ownerModelClazz.getName() + " to " + associatedModelName);
			throw new ClassNotFoundException();
		}
		
		try {
			String fieldName = StringUtilities.uncapitalize(associatedModelName);
			boolean isManyToOne = persistence.isManyToOne(ownerModelClazz, fieldName);
			boolean isOneToMany = persistence.isOneToMany(ownerModelClazz, fieldName);

			/*
			 * If we don't have any annotations, assume the
			 * requested association doesn't exist
			 */
			if (!isManyToOne && !isOneToMany) {
				log.warn("Could not find " + type + " association from " + 
						ownerModelClazz.getName() + " to " + associatedModelName);
				throw new ClassNotFoundException();
			}

			this.type = isManyToOne ? "belongsTo" : "hasMany";
			
			if (this.type.equals("belongsTo")) {
				associatedModelClazz = getter.getReturnType();
			} else {
				ParameterizedType pt = (ParameterizedType) getter.getGenericReturnType();
				associatedModelClazz = (Class<?>) pt.getActualTypeArguments()[0];
			}
			
		
			if (associatedModelPrefix.isEmpty()) {
				/*
				 * The associated model name begins with the parent model name
				 */
				String ownerModelName = ownerModelDefinition.getModelName();
				if (ownerModelName.contains(".model.")) ownerModelName = ownerModelName.split(".model.")[1];
				this.modelName = ownerModelName + associatedModelName;
				
			} else {
				this.modelName = associatedModelPrefix + associatedModelName;
			}
			
			this.name = StringUtilities.uncapitalize(associatedModelName);
			
			/**
			 * TODO:  Reevaluate this entire section
			 */
			if (this.type.equals("belongsTo")) {
				this.foreignKey = this.name + "Id";
				this.associationKey = this.name;
			}
			
			
		} catch (Exception e) {
			log.warn("Error", e);
		}
		
		
		/*
		 * Create a new model definition for this association
		 */
		this.associatedModelDefinition = new ModelDefinition(this);
		
	}
	
	@JsonIgnore
	public Config getConfig() {
		return config;
	}
	
	@JsonIgnore
	public Class<?> getAssociatedModelClazz() {
		return associatedModelClazz;
	}
	
	@JsonIgnore
	public ModelDefinition getAssociatedModelDefinition() {
		return associatedModelDefinition;
	}

	
//	@JsonIgnore
//	public ModelDefinition getAssociatedModelName() {
//		return associatedModelName;
//	}
	
	@JsonIgnore
	public Class<?> getOwnerModelClazz() {
		return ownerModelClazz;
	}
	
	@JsonIgnore
	public ModelDefinition getOwnerModelDefinition() {
		return ownerModelDefinition;
	}
	
	/*
	 * This is a duplicate getter to retrieve model name.  The model
	 * name is actually stored in the model field so Jackson can
	 * easily serialize this class with model:"ModelName" and also
	 * maintain consistency by providing a getter @ getModelName()
	 */
//	@JsonIgnore
//	public String getModelName() {
//		return model;
//	}
	
	@JsonInclude(JsonInclude.Include.NON_NULL)
	public String getType() {
		return type;
	}
	
	@JsonInclude(JsonInclude.Include.NON_NULL)
	@JsonProperty("model")
	public String getModelName() {
		return modelName;
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public String getForeignKey() {
		return foreignKey;
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public String getAssociationKey() {
		return associationKey;
	}	
	@JsonInclude(JsonInclude.Include.NON_NULL)
	public String getName() {
		return name;
	}
}
