package com.adaptrex.core.ext.data;

import com.adaptrex.core.persistence.AdaptrexPersistence;
import com.adaptrex.core.service.CdiService;
import com.fasterxml.jackson.annotation.JsonInclude;

public class FieldDefinition {

	private String		name;
	private String		type;
	private Boolean		useNull;
	private String		dateFormat;
	
	private static AdaptrexPersistence persistence = CdiService.getAdaptrexPersistence();
	
	public FieldDefinition(String name, String type, boolean useNull) {
		this.name = name;
		this.type = type;
		this.useNull = useNull;
	}
	
	public FieldDefinition(java.lang.reflect.Field field) {
		/*
		 * Get the field name
		 */
		this.name = field.getName();

		
		/*
		 * Get the type.  If it's a join column set it to auto
		 */
		String columnType = field.getType().getSimpleName().toLowerCase();
		
		if (columnType.equals("integer") || columnType.equals("long")) {
			columnType = "int";
		}

		else if (columnType.equals("double")) {
			columnType = "float";
		}
		
		else if (columnType.equals("time")) {
			columnType = "date";
			this.dateFormat = "H:i:s";
		}
		
		else if (columnType.equals("calendar") || columnType.equals("date")) {
			columnType = "date";
			this.dateFormat = "Y-m-d";
		}

		this.type = columnType;
		
		/*
		 * Ext always wants "null" for number fields otherwise it generates a 0
		 */
		if (this.type.equals("int") || this.type.equals("float")) this.useNull = true;
	}
	
	
	
	
	@JsonInclude
	public String getName() {
		return this.name;
	}
	
	@JsonInclude
	public String getType() {
		return this.type;
	}
	
	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Boolean getUseNull() {
		return this.useNull;
	}
	
	@JsonInclude(JsonInclude.Include.NON_NULL)
	public String getDateFormat() {
		return this.dateFormat;
	}
}
