package com.adaptrex.core.ext.data;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.annotation.JsonInclude;

/*
 * This class is generally only used to serialize the json for a store definition
 */
public class StoreDefinition implements Serializable {
	private static final long serialVersionUID = -1L;

	@JsonInclude public final String extend = "Ext.data.Store";
	
	private Logger log = LoggerFactory.getLogger(StoreDefinition.class);
	
	private Config	config;
	private Store	store;
	
	protected StoreDefinition() {}

	protected StoreDefinition(Store store) {
		this.store = store;
		this.config = store.getConfig();
	}
	

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public List<Map<String,Object>> getData() {
		if (!config.isInline()) return null;
		return store.getData();
	}
	
	
	/*
	 * We always return the name of the model
	 */
	@JsonInclude
	public String getModel() {
		return (config.getNamespace() == null) 
				? config.getModelName()
				: config.getNamespace() + ".model." + config.getModelName();
	}
	
	/*
	 * Below are just a list of getters and setters including
	 * information to help jackson serialize them properly
	 */
	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Integer getPageSize() {
		return config.getPageSize();
	}
	
	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Boolean isClearOnPageLoad() {
		return config.getClearOnPageLoad();
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Boolean isClearRemovedOnLoad() {
		return config.getClearRemovedOnLoad();
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Boolean isAutoSync() {
		return config.getAutoSync();
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Boolean isBuffered() {
		return config.getBuffered();
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Object getAutoLoad() {
		return config.getAutoLoad();
	}
	
	@JsonInclude(JsonInclude.Include.NON_EMPTY)
	public List<Sorter> getGroupers() {
		return config.getGroupers();
	}
	
	@JsonInclude(JsonInclude.Include.NON_EMPTY)
	public List<Sorter> getSorters() {
		return config.getSorters();
	}
	
	@JsonInclude(JsonInclude.Include.NON_EMPTY)
	public List<Filter> getFilters() {
		return config.getFilters();
	}
	
	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Integer getLeadingBufferZone() {
		return config.getLeadingBufferZone();
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Integer getTrailingBufferZone() {
		return config.getTrailingBufferZone();
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Boolean isRemoteGroup() {
		return config.getRemoteGroup();
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Boolean isRemoteSort() {
		return config.getRemoteSort();
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Boolean isRemoteFilter() {
		return config.getRemoteFilter();
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Boolean isSortOnFilter() {
		return config.getSortOnFilter();
	}

	@JsonInclude(JsonInclude.Include.NON_NULL)
	public Integer getPurgePageCount() {
		return config.getPurgePageCount();
	}
} 
