package com.adaptrex.core.persistence.jpa;

import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.xml.parsers.SAXParserFactory;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

import com.adaptrex.core.ext.rest.ExtConfig;

@ApplicationScoped
public class JpaPersistenceManager {
	
	public JpaPersistenceManager() {} 
	
	private Map<String, EntityManagerFactory> entityManagerFactories = new HashMap<String, EntityManagerFactory>();
	private EntityManagerFactory defaultFactory = null;
	
	private Logger log = LoggerFactory.getLogger(JpaPersistenceManager.class);
	
	public EntityManagerFactory getEntityManagerFactory() {
		return defaultFactory;
	}
	
	@PostConstruct
	public void initialize() {
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		InputStream inputStream = cl.getResourceAsStream("META-INF/persistence.xml");

		if (inputStream == null) {
			log.warn("Could not load default entity manager factory:  persistence.xml could not be found");
			return;
		}
		
		try {
			SAXParserFactory.newInstance().newSAXParser().parse(inputStream, new DefaultHandler() {
			    public void startElement(String uri, String localName, String qName, Attributes attributes) {
			    	if (!qName.equalsIgnoreCase("persistence-unit") || defaultFactory != null) return;
			    	
			    	String persistenceUnitName = attributes.getValue("name");
			    	log.debug("Creating JPA Persistence Unit: " + persistenceUnitName);
		    		defaultFactory = Persistence.createEntityManagerFactory(persistenceUnitName);
		    		entityManagerFactories.put(persistenceUnitName, defaultFactory);
				}
			});
		} catch (Exception e) {
			log.warn("Error", e);
		}
	}
	
	
	/*
	 * Shutdown Persistence Manager
	 */
	@PreDestroy
	public void destroy() {
		try {
			for (String key : entityManagerFactories.keySet()) {
				entityManagerFactories.get(key).close();
				log.info("Closing EntityManagerFactory for: " + key);
			}
		} catch (Exception e) {
			log.warn("Error Closing EntityManagerFactory", e);
		}
	}
}
