package com.adaptrex.core.security.interceptor;

import javax.interceptor.AroundInvoke;
import javax.interceptor.Interceptor;
import javax.interceptor.InvocationContext;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adaptrex.core.security.annotation.SecureAuthentication;

@Interceptor
@SecureAuthentication
public class SecureAuthenticationInterceptor {

	private static Logger logger = LoggerFactory.getLogger(SecureAuthenticationInterceptor.class);
	
	@AroundInvoke
    public Object checkSecure(InvocationContext ctx) throws Exception {
		Subject subject = SecurityUtils.getSubject();
		if (!subject.isAuthenticated()) {
			logger.info("Not Authenticated for: Anonymous @ " + 
					ctx.getMethod().getDeclaringClass() + "." + ctx.getMethod().getName() + "()");
			return null;
		}
		
		return ctx.proceed();
    }
}