package com.adaptrex.core.security.interceptor;

import javax.interceptor.AroundInvoke;
import javax.interceptor.Interceptor;
import javax.interceptor.InvocationContext;

import org.apache.shiro.SecurityUtils;
import org.apache.shiro.subject.Subject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adaptrex.core.security.annotation.SecureGuest;

@Interceptor
@SecureGuest
public class SecureGuestInterceptor {

	private static Logger log = LoggerFactory.getLogger(SecureGuestInterceptor.class);
	
	@AroundInvoke
    public Object checkSecure(InvocationContext ctx) throws Exception {
		Subject subject = SecurityUtils.getSubject();
		if (subject.isAuthenticated()) {
			log.debug("Guest Only Not Authorized for: " + subject.getPrincipal().toString() + " @ " + 
					ctx.getMethod().getDeclaringClass() + "." + ctx.getMethod().getName() + "()");
			return null;
		}
		
		return ctx.proceed();
    }
}
