package com.adaptrex.core.service;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.spi.Bean;
import javax.enterprise.inject.spi.BeanManager;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adaptrex.core.config.AdaptrexConfig;
import com.adaptrex.core.config.ContextConfig;
import com.adaptrex.core.persistence.AdaptrexPersistence;

public class CdiService {
	private static Logger log = LoggerFactory.getLogger(CdiService.class);

	private static BeanManager beanManager;
	public static BeanManager getBeanManager() {
		if (beanManager != null) return beanManager;
		
        try{
            InitialContext initialContext = new InitialContext();
            return (BeanManager) initialContext.lookup("java:comp/env/WeldBeanManager");
        } catch (NamingException e) {
        	log.warn("Error", e);
            return null;
        }
	}
	
	private static AdaptrexPersistence adaptrexPersistence;
	public static AdaptrexPersistence getAdaptrexPersistence() {
		if (adaptrexPersistence != null) return adaptrexPersistence;
		
		BeanManager beanManager = getBeanManager();
		
		@SuppressWarnings("unchecked")
		Bean<AdaptrexPersistence> bean = (Bean<AdaptrexPersistence>) beanManager.getBeans(AdaptrexPersistence.class).iterator().next();
        CreationalContext<AdaptrexPersistence> ctx = beanManager.createCreationalContext(bean);
        adaptrexPersistence = (AdaptrexPersistence) beanManager.getReference(bean, AdaptrexPersistence.class, ctx);
        return adaptrexPersistence;
	}
	
	private static ContextConfig contextConfig;
	public static ContextConfig getContextConfig() {
		if (contextConfig != null) return contextConfig;
		BeanManager beanManager = getBeanManager();
		@SuppressWarnings("unchecked")
		Bean<ContextConfig> bean = (Bean<ContextConfig>) beanManager.getBeans(ContextConfig.class).iterator().next();
        CreationalContext<ContextConfig> ctx = beanManager.createCreationalContext(bean);
        contextConfig = (ContextConfig) beanManager.getReference(bean, ContextConfig.class, ctx);
        return contextConfig;
	}
	
	private static AdaptrexConfig adaptrexConfig;
	public static AdaptrexConfig getAdaptrexConfig() {
		if (adaptrexConfig != null) return adaptrexConfig;
		BeanManager beanManager = getBeanManager();
		@SuppressWarnings("unchecked")
		Bean<AdaptrexConfig> bean = (Bean<AdaptrexConfig>) beanManager.getBeans(AdaptrexConfig.class).iterator().next();
        CreationalContext<AdaptrexConfig> ctx = beanManager.createCreationalContext(bean);
        adaptrexConfig = (AdaptrexConfig) beanManager.getReference(bean, AdaptrexConfig.class, ctx);
        return adaptrexConfig;
	}
}
