package com.adaptrex.core.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;
import javax.enterprise.context.ApplicationScoped;
import javax.inject.Inject;
import javax.mail.BodyPart;
import javax.mail.Message;
import javax.mail.Multipart;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.naming.InitialContext;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.adaptrex.core.config.AdaptrexConfig;
import com.adaptrex.core.config.ContextConfig;
import com.adaptrex.core.utilities.StringUtilities;

@ApplicationScoped
public class Mail {

	private static final Logger log = LoggerFactory.getLogger(Mail.class);
	
	private @Inject ContextConfig contextConfig;
	private @Inject AdaptrexConfig adaptrexConfig;
	private static Map<String,Session> mailSessions = new HashMap<String,Session>();
	
	private Session getSession() {
		return getSession(null);
	}
	
	@SuppressWarnings("unchecked")
	private Session getSession(String name) {
		if (name == null) name = "_default";
		
		Map<String,Object> config = (Map<String, Object>) contextConfig.get("smtp");
		if (config == null) config = (Map<String, Object>) adaptrexConfig.get("smtp");

		final String userName = (String) config.get("user");
		final String password = (String) config.get("password");
		
		Properties props = new Properties();
		if (config.get("properties") != null) {
			Map<String,Object> propMap = (Map<String,Object>) config.get("properties");
			for (String propKey : propMap.keySet()) {
				props.put("mail." + propKey, propMap.get(propKey));
			}
		}
		if (config.get("host") != null) 	props.put("mail.smtp.host",	config.get("host"));
		if (config.get("from") != null) 	props.put("mail.from",		config.get("from"));
		if (config.get("port") != null) 	props.put("mail.smtp.port", String.valueOf(config.get("port")));
		
		if ((Boolean) config.get("auth"))	props.put("mail.smtp.auth", "true");
		if ((Boolean) config.get("debug"))	props.put("mail.debug", "true");
		if ((Boolean) config.get("ssl")) {
			props.put("mail.smtp.socketFactory.port", String.valueOf(config.get("port")));
			props.put("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
			props.put("mail.smtp.socketFactory.fallback", "false");
		}
		
		Session session = Session.getDefaultInstance(props,
			new javax.mail.Authenticator() {
				protected PasswordAuthentication getPasswordAuthentication() {
					return new PasswordAuthentication(userName, password);
				}
			});
		mailSessions.put("_default", session);
		
		return session;
	}
	
	
	public boolean send(String subject, String body, String sendTo) {
		List<String> sendToList = new ArrayList<String>();
		sendToList.add(sendTo);
		return send(subject, body, sendToList);
	}
	
	public boolean send(String subject, String body, List<String> sendTo) {
		try {
		    Message msg = createMessage();
		    msg.setSubject(subject);
		    msg.setContent(body, "text/html");
		    msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(StringUtilities.join(sendTo, ",")));
		    Transport.send(msg);
		} catch (Exception e) {
			log.warn("Error", e);
			return false;
		}
	    return true;
	}
	
	
	/*
	 * Send Attachments
	 */
	public boolean sendWithAttachments(String subject, String body, String attachPath, String sendTo) {
		List<String> sendToList = new ArrayList<String>();
		sendToList.add(sendTo);
		List<String> attachPathList = new ArrayList<String>();
		sendToList.add(attachPath);
		return sendWithAttachments(subject, body, attachPathList, sendToList);
	}

	public boolean sendWithAttachments(String subject, String body, List<String> attachPathList, String sendTo) {
		List<String> sendToList = new ArrayList<String>();
		sendToList.add(sendTo);
		return sendWithAttachments(subject, body, attachPathList, sendToList);
	}
	
	public boolean sendWithAttachments(String subject, String body, String attachPath, List<String> sendToList) {
		List<String> attachPathList = new ArrayList<String>();
		sendToList.add(attachPath);
		return sendWithAttachments(subject, body, attachPathList, sendToList);
	}
	
	private boolean sendWithAttachments(String subject, String body, List<String> attachPath, List<String> sendTo) {
		try {
		    Message msg = createMessage();
		    msg.setSubject(subject);
		    msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(StringUtilities.join(sendTo, ",")));
		    
		    Multipart multipart = new MimeMultipart();
		    
		    BodyPart messageBodyPart = new MimeBodyPart();
		    messageBodyPart.setText(body);
		    multipart.addBodyPart(messageBodyPart); 

		    for (String path : attachPath) {
		    	messageBodyPart = new MimeBodyPart();
				DataSource source = new FileDataSource(path);
				messageBodyPart.setDataHandler(new DataHandler(source));
				messageBodyPart.setFileName(path);
				multipart.addBodyPart(messageBodyPart);
		    }
			
			msg.setContent(multipart);
		    
		    Transport.send(msg);
		} catch (Exception e) {
			log.warn("Error", e);
			return false;
		}
	    
	    return true;		
	}
	
	public Message createMessage() {
		try {
		    Message msg = new MimeMessage(getSession());
		    msg.setHeader("X-Mailer", "Adaptrex Mailer");
		    return msg;
		} catch (Exception e) {
			log.warn("Error", e);
			return null;
		}
	}
}
